//  **** BEGIN LICENSE BLOCK ****
//  Copyright(c) 2005-2006 Adam Judson
//
//
//  PhotoMap
//  - display user contributed photos in a map
//
//  This program is free software; you can redistribute it and/or modify it under
//  the terms of the GNU General Public License as published by the Free
//  Software Foundation; either version 2 of the License, or (at your option)
//  any later version.
//
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
//  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
//  more details.
//
//  You should have received a copy of the GNU General Public License along with
//  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
//  Place, Suite 330, Boston, MA 02111-1307 USA
//
//  **** END LICENSE BLOCK ****


// the wrapper for our specific map implementation
function PhotoMapFPhoto() {
   PhotoMapBasePhoto.prototype.init.call(this);
   this.init();
}

PhotoMapFPhoto.prototype = {
   __proto__ : PhotoMapBasePhoto.prototype,

   init : function() {
      this.id = this.preferences.getFlickrId();
      this.authenticationCheck();
      this.photoInfoQueue = new TimedQueue(100, 4, new Callback(this, this.getPhotoInfo), new Callback(this, this.canSendTransactions));
   },

   searchPhotos : function(searchTags, callback) {
      searchTags.push("PhotoMap");
      searchTags.push("pmVer01");
      var txn = new SearchPhotoTransaction(new Callback(this, this.doneSearch), searchTags);
      txn.callback = callback;
      txn.send();
   },

   doneSearch : function(txn, wasError) {
      if (wasError) {
         PhotoMapUtils.log("There was an error with search: [" + txn.getURL() + "] - " + wasError);
      } else {
         var txnPhotos = txn.getPhotoSet();
         // the call back will set the search field if we don't have this
         // photo already
         txn.callback.execute(txnPhotos)
         this.parseSearchResults(txnPhotos);
      }
   },

   parseSearchResults : function(txnPhotos) {
      // now for each of these, do an info transaction
      // this should really be part of a composite transaction...
      var photoIndex, photo;
      for (photoIndex = 0; photoIndex < txnPhotos.length; photoIndex++) {
         photo = txnPhotos[photoIndex];
         if (photo.search) {
            PhotoMapUtils.log("Need info for photo: " + photo.id);
            this.photoInfoQueue.add(photo);
         } else {
            this.controller.photoUpdate(photo);
         }
      }
   }, 

   getPhotoInfo : function(photo) {
      var infoTxn = new PhotoTagListTransaction(new Callback(this, this.doneInfo), photo);
      infoTxn.send();
   },

   doneInfo : function(txn, wasError) {
      if (wasError) {
         PhotoMapUtils.log("There was an error with info: [" + txn.getURL() + "] - " + wasError);
      } else {
         var photo = txn.getPhoto();
         // does it have enough info?
         if (photo.version && photo.lat && photo.lon) {
            this.controller.photoUpdate(photo);
         } else {
            PhotoMapUtils.log("Not enough info in photo: [" + photo.id +"]");
         }
      }
   },

   authenticationCheck : function() {
      this.totalPictures = "?";
      this.authenticated = false;
      if (!this.token) {
         this.token = this.preferences.getFlickrToken();
         if (this.token) {
            var txn = new FlickrCheckTokenTransaction(new Callback(this, this.doneTokenCheck), this.token);
            txn.send();
         }
      }
   },

   doneTokenCheck : function(txn, wasError) {
      if (wasError) {
         PhotoMapUtils.log("There was an error with tokenCheck: [" + txn.getURL() + "] - " + wasError);
      } else {
         var perms = txn.getPerms();
         this.authenticated = (perms == "write");
         if (this.authenticated) {
            this.username = txn.getUserName();
            this.id = txn.getId();
            this.getPictureCount();
         }
      }
   },

   authenticate : function() {
      this.token = "";
      this.authenticated = false;
      var frobTxn = new FlickrGetFrobTransaction(new Callback(this, this.doneFrob));
      frobTxn.send();
   },

   doneFrob : function(txn, wasError) {
      if (wasError) {
         PhotoMapUtils.log("There was an error with getFrom: [" + txn.getURL() + "] - " + wasError);
      } else {
         this.frob = txn.getFrob();
         this.controller.openWindow(txn.getAuthenticationUrl());
      }
   },

   // called externally when button is pressed
   doneAuthenticate : function() {
      // check the frob is okay first...
      var tokenTxn = new FlickrGetTokenTransaction(new Callback(this, this.doneToken), this.frob);
      tokenTxn.send();
   },

   doneToken : function(txn, wasError) {
      if (wasError) {
         PhotoMapUtils.log("There was an error with token retrieval: [" + txn.getURL() + "] - " + wasError);
      } else {
         this.token = txn.getToken();
         if (txn.getPerms() == "write") {
            this.preferences.setFlickrToken(this.token);
            this.authenticated = true;
            this.username = txn.getUserName();
            this.id = txn.getId();
            this.getPictureCount();
            this.controller.authenticationSuccess();
         }
      }
   },

   getPictureCount : function() {
      var txn = new FlickrPeopleInfoTransaction(new Callback(this, this.donePeopleInfo), this.id);
      txn.send();
   },

   donePeopleInfo : function(txn, wasError) {
      if (wasError) {
         PhotoMapUtils.log("There was an error with people info: [" + txn.getURL() + "] - " + wasError);
      } else {
         this.totalPictures = txn.getCount();
      }
   },

   getNewAccountUrl : function() {
      return this.preferences.getFlickrNewAccountUrl();
   },

   needsAuthentication : function() {
      return !this.authenticated;
   },

   logout : function() {
      this.token = "";
      this.preferences.setFlickrToken(this.token);
      this.authenticated = false;
   },

   getUserName : function() {
      return this.username;
   },

   getTotalPictures : function() {
      return this.totalPictures;
   },

   upload : function(editInfo, callback) {
      var txn = new FlickrUploadTransaction(new Callback(this, this.doneUpload), editInfo);
      txn.callback = callback;
      txn.send();
   },

   doneUpload : function(txn, wasError) {
      if (wasError) {
         PhotoMapUtils.log("There was an error with upload: [" + txn.getURL() + "] - " + wasError);
         txn.callback.execute(wasError);
      } else {
         PhotoMapUtils.log("Upload is done.\n\nTicket Id: " + txn.getTicketId());
         txn.callback.execute();
      }
   },

   canSendTransactions : function(queue) {
      return BaseFlickrPhotoTransaction.transactionsPerSecond() < 0.90;
   }
}
