//  **** BEGIN LICENSE BLOCK ****
//  Copyright(c) 2005-2006 Adam Judson
//
//
//  PhotoMap
//  - display user contributed photos in a map
//
//  This program is free software; you can redistribute it and/or modify it under
//  the terms of the GNU General Public License as published by the Free
//  Software Foundation; either version 2 of the License, or (at your option)
//  any later version.
//
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
//  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
//  more details.
//
//  You should have received a copy of the GNU General Public License along with
//  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
//  Place, Suite 330, Boston, MA 02111-1307 USA
//
//  **** END LICENSE BLOCK ****


// our transaction object(s)
function PhotoMapPhotoTransaction(callback) {
   // make sure we get our function
   PhotoMapPhotoTransaction.prototype.init.call(this, callback);
}

PhotoMapPhotoTransaction.totalTransactions = 0;

PhotoMapPhotoTransaction.prototype = {
   init : function(callback) {
      this.internalCallback = callback;
      this.preferences = new PhotoMapPreferences();
      this.request = new XMLHttpRequest();
   },

   getMethod : function() {
      return "GET";
   },

   getURL : function() {
      return this.url;
   },

   send : function() {
      PhotoMapPhotoTransaction.totalTransactions++;
      this.request.open(this.getMethod(), this.getURL(), true);
      this.setRequestHeaders();
      var txn = this;
      this.request.onreadystatechange = function() {
                                    		if (txn.request.readyState == 4) {
                                             txn.done();
                                          }
                                    	};
      try {
         this.request.send(this.getBody());
      } catch (e) {
         PhotoMapUtils.log("Problem with transaction: " + this.getMethod() + ", " + this.getURL() + ", " + e);
         delete this.request;
      }
   },

   setRequestHeaders : function() {
      var headers = this.getRequestHeaders();
      if (headers) {
         for (header in headers) {
            this.request.setRequestHeader(header, headers[header]);
         }
      }
   },

   getRequestHeaders : function() {
      return undefined;
   },

   body : null,
   getBody : function() {
      return this.body;
   },

   // called by transaction listener
   done : function() {
      this.xmlResponse = this.request.responseXML;
      var error = false;
      try {
         this.parseResponse();
      } catch (e) {
         PhotoMapUtils.log("Problem with transaction: " + e);
         error = e;
      }
      delete this.xmlResponse;
      this.internalCallback.execute(this, error);
   },

   // called externally - need to think about this more, not really used right now
   abort : function() {
      if (this.request) {
         this.request.abort();
      }
      this.callback.execute(this, "abort");
   },

   // xml helper functions
   getStringValue : function(key, node) {
      return document.evaluate(key, node, null, XPathResult.STRING_TYPE, null).stringValue;
   },

   getNodes : function(text) {
      return document.evaluate(text, this.xmlResponse, null, XPathResult.ORDERED_NODE_ITERATOR_TYPE, null);   
   }
}


function BaseFlickrPhotoTransaction(callback) {
   PhotoMapPhotoTransaction.call(this, callback);
   // make sure we get our function
   BaseFlickrPhotoTransaction.prototype.init.call(this);
}

BaseFlickrPhotoTransaction.totalTransactions = 0;
BaseFlickrPhotoTransaction.startTime = (new Date()).getTime();

BaseFlickrPhotoTransaction.transactionsPerSecond = function() {
   var currentTime = (new Date()).getTime();
   var elapsedTime = currentTime - BaseFlickrPhotoTransaction.startTime;
   var transactionsPerSecond = Math.round((BaseFlickrPhotoTransaction.totalTransactions/(elapsedTime/1000)) * 100)/100;
   return transactionsPerSecond;
}

BaseFlickrPhotoTransaction.prototype = {
   __proto__ : PhotoMapPhotoTransaction.prototype,

   init : function() {
   },

   send : function() {
      BaseFlickrPhotoTransaction.totalTransactions++;
      PhotoMapPhotoTransaction.prototype.send.call(this);
   },

   checkForErrors : function() {
      if (this.xmlResponse) {
         // do something smart here
         var status = this.getStringValue("@stat", this.getNodes("/rsp", this.xmlResponse).iterateNext());
         if (status != "ok") {
            var code = this.getStringValue("@code", this.getNodes("/rsp/err", this.xmlResponse).iterateNext());
            var msg  = this.getStringValue("@msg", this.getNodes("/rsp/err", this.xmlResponse).iterateNext());
            throw "Failure: " + code + " [" + msg + "]";
         }
      } else {
         throw "No response";
      }
   }
}

function SearchPhotoTransaction(callback, tags) {
   BaseFlickrPhotoTransaction.call(this, callback);
   this.init(tags);
}

SearchPhotoTransaction.prototype = {
   __proto__ : BaseFlickrPhotoTransaction.prototype,
   baseURL : "http://www.flickr.com/services/rest/?method=flickr.photos.search&api_key=",
   baseURL2 : "&tag_mode=all&tags=",

   init : function(tags) {
      this.tags = tags;
      var url = this.baseURL + this.preferences.getFlickrId() + this.baseURL2;
      var first = true;
      for (var tagIndex = 0; tagIndex < tags.length; tagIndex++) {
         if (first) {
            first = false;
         } else {
            url += "+";
         }
         url += this.fixTag(tags[tagIndex]);
      }
      this.url = url;
   },

   fixTag : function(tag) {
      // don't send decimals, or minus signs or colons.
      var newTag = tag.replace(/-/g, "").replace(/\./g, "").replace(/:/g, "");
      return newTag;
   },

   parseResponse : function() {
      // this throws an exception...
      this.checkForErrors();

      var imageArray = new Array();

      // pull the values from xmlresponse
      var tmpId, tmpOwner, tmpServer, tmpSecret, tmpTitle, tmpURL;
      var nodes = this.getNodes("//photo");    
      var currentNode = nodes.iterateNext();
      while (currentNode) {
         tmpId     = this.getStringValue("@id",     currentNode);
         tmpOwner  = this.getStringValue("@owner",  currentNode);
         tmpServer = this.getStringValue("@server", currentNode);
         tmpSecret = this.getStringValue("@secret", currentNode);
         tmpTitle  = this.getStringValue("@title",  currentNode);

         PhotoMapUtils.log("Parsing a photo: " + tmpId);
         imageArray.push(new FlickrPhoto(tmpTitle, tmpId, tmpSecret, tmpOwner, tmpServer));
         currentNode = nodes.iterateNext();
      }
      this.photoSet = imageArray;
      // not really a usefull image array at this point, 
      // we still need to do a query on the individual pictures      
   },

   getPhotoSet : function() {
      return this.photoSet;
   }
}

function PhotoInfoTransaction(callback, photo) {
   BaseFlickrPhotoTransaction.call(this, callback);
   this.init(photo);
}

PhotoInfoTransaction.prototype = {
   __proto__ : BaseFlickrPhotoTransaction.prototype,
   baseURL : "http://www.flickr.com/services/rest/?method=flickr.photos.getInfo&api_key=",

   init : function(photo) {
      if (!photo || !photo.id || !photo.secret) {
         throw "Bad Photo";
      }
      this.photo = photo;
      var url = this.baseURL + this.preferences.getFlickrId();
      url += "&photo_id=" + this.photo.id;
      url += "&secret=" + this.photo.secret;
      this.url = url;
   },

   parseResponse : function() {
      // this throws an exception...
      this.checkForErrors();

      // pull the values from xmlresponse
      var url = this.getStringValue("//url", this.xmlResponse);
      this.photo.url = url;

      var nodes = this.getNodes("//tag");   
      var tag, rawTag; 
      var currentNode = nodes.iterateNext();
      while (currentNode) {
         tag = currentNode.textContent;
         rawTag = this.getStringValue("@raw", currentNode);

         this.photo.setValue(tag, rawTag);

         currentNode = nodes.iterateNext();
      }

      // add date/time information so we can display by
      // season, time of day etc.

   },

   getPhoto : function() {
      return this.photo;
   }
}

function PhotoTagListTransaction(callback, photo) {
   BaseFlickrPhotoTransaction.call(this, callback);
   this.init(photo);
}

PhotoTagListTransaction.prototype = {
   __proto__ : BaseFlickrPhotoTransaction.prototype,
   baseURL : "http://www.flickr.com/services/rest/?method=flickr.tags.getListPhoto&api_key=",

   init : function(photo) {
      if (!photo || !photo.id || !photo.secret) {
         throw "Bad Photo";
      }
      this.photo = photo;
      var url = this.baseURL + this.preferences.getFlickrId();
      url += "&photo_id=" + this.photo.id;
      this.url = url;
   },

   parseResponse : function() {
      // this throws an exception...
      this.checkForErrors();

      // pull the values from xmlresponse
      var nodes = this.getNodes("//tag");   
      var tag, rawTag; 
      var currentNode = nodes.iterateNext();
      while (currentNode) {
         tag = currentNode.textContent;
         rawTag = this.getStringValue("@raw", currentNode);

         this.photo.setValue(tag, rawTag);

         currentNode = nodes.iterateNext();
      }
   },

   getPhoto : function() {
      return this.photo;
   }
}

// wtf does FROB stand for?
function FlickrGetFrobTransaction(callback) {
   BaseFlickrPhotoTransaction.call(this, callback);
   this.init();
}

FlickrGetFrobTransaction.prototype = {
   __proto__ : BaseFlickrPhotoTransaction.prototype,
   baseURL : "http://www.flickr.com/services/rest/?method=flickr.auth.getFrob&api_key=",

   init : function() {
      var url = this.baseURL + this.preferences.getFlickrId();
      var params = this.preferences.getFlickrSecret() + "api_key" + this.preferences.getFlickrId() + "method" + "flickr.auth.getFrob";
      var signature = PhotoMapUtils.getMD5(params);
      url += "&api_sig=" + signature;
      this.url = url;
   },

   parseResponse : function() {
      // this throws an exception...
      this.checkForErrors();

      // pull the values from xmlresponse
      this.frob = this.getStringValue("//frob", this.xmlResponse);

      var apiKey = this.preferences.getFlickrId();

      var authenticateParams = this.preferences.getFlickrSecret() + 
                                "api_key" + apiKey + 
                                "frob" + this.frob +
                                "perms" + "write";

      var signature = PhotoMapUtils.getMD5(authenticateParams);
      this.authenticateUrl = "http://flickr.com/services/auth/?api_key=" + apiKey +
                             "&perms=" + "write" + 
                             "&frob=" + this.frob +
                             "&api_sig=" + signature;
   },

   getFrob : function() {
      return this.frob;
   },

   getAuthenticationUrl : function() {
      return this.authenticateUrl;
   }
}

function FlickrGetTokenTransaction(callback, frob) {
   BaseFlickrPhotoTransaction.call(this, callback);
   this.init(frob);
}

FlickrGetTokenTransaction.prototype = {
   __proto__ : BaseFlickrPhotoTransaction.prototype,
   baseURL : "http://www.flickr.com/services/rest/?method=flickr.auth.getToken&api_key=",

   init : function(frob) {
      var url = this.baseURL + this.preferences.getFlickrId();
      var params = this.preferences.getFlickrSecret() + 
                   "api_key" + this.preferences.getFlickrId() + 
                   "frob" + frob +
                    "method" + "flickr.auth.getToken";
      var signature = PhotoMapUtils.getMD5(params);
      url += "&frob=" + frob + "&api_sig=" + signature;
      this.url = url;
   },

   parseResponse : function() {
      // this throws an exception...
      this.checkForErrors();

      // pull the values from xmlresponse
      this.token = this.getStringValue("//token", this.xmlResponse);
      this.perms = this.getStringValue("//perms", this.xmlResponse);
      var nodes = this.getNodes("//user");
      var firstNode = nodes.iterateNext();
      this.username = this.getStringValue("@username", firstNode);
      this.id       = this.getStringValue("@nsid", firstNode);
   },

   getToken : function() {
      return this.token;
   },

   getPerms : function() {
      return this.perms;
   },

   getId : function() {
      return this.id;
   },

   getUserName : function() {
      return this.username;
   }
}

function FlickrCheckTokenTransaction(callback, token) {
   BaseFlickrPhotoTransaction.call(this, callback);
   this.init(token);
}

FlickrCheckTokenTransaction.prototype = {
   __proto__ : BaseFlickrPhotoTransaction.prototype,
   baseURL : "http://www.flickr.com/services/rest/?method=flickr.auth.checkToken&api_key=",

   init : function(token) {
      var url = this.baseURL + this.preferences.getFlickrId() +
                "&auth_token=" + token;
      this.url = url;
   },

   parseResponse : function() {
      // this throws an exception...
      this.checkForErrors();

      // pull the values from xmlresponse
      this.perms = this.getStringValue("//perms", this.xmlResponse);
      var nodes = this.getNodes("//user");
      var firstNode = nodes.iterateNext();
      this.username = this.getStringValue("@username", firstNode);
      this.id       = this.getStringValue("@nsid", firstNode);
   },

   getPerms : function() {
      return this.perms;
   },

   getUserName : function() {
      return this.username;
   },

   getId : function() {
      return this.id;
   }
}

function FlickrPeopleInfoTransaction(callback, id) {
   BaseFlickrPhotoTransaction.call(this, callback);
   this.init(id);
}

FlickrPeopleInfoTransaction.prototype = {
   __proto__ : BaseFlickrPhotoTransaction.prototype,
   baseURL : "http://www.flickr.com/services/rest/?method=flickr.people.getInfo&api_key=",

   init : function(id) {
      var url = this.baseURL + this.preferences.getFlickrId() +
                "&user_id=" + id;
      this.url = url;
   },

   parseResponse : function() {
      // this throws an exception...
      this.checkForErrors();

      // pull the values from xmlresponse
      this.count = this.getStringValue("//count", this.xmlResponse);
   },

   getCount : function() {
      return this.count;
   }
}

function FlickrUploadTransaction(callback, editInfo) {
   BaseFlickrPhotoTransaction.call(this, callback);
   this.init(editInfo);
}

FlickrUploadTransaction.prototype = {
   __proto__ : BaseFlickrPhotoTransaction.prototype,
   baseURL : "http://www.flickr.com/services/upload/",

   getMethod : function() {
      return "POST";
   },

   init : function(editInfo) {
      this.editInfo = editInfo;
      this.url = this.baseURL;
      var tags = FlickrPhoto.getTags(editInfo);
      var description = FlickrPhoto.getDescription(editInfo);
      var id = this.preferences.getFlickrId();
      var token = this.preferences.getFlickrToken();
      var params = this.preferences.getFlickrSecret() + 
                   "api_key" + id + 
                   "async"   + "1" + 
                   "auth_token" + token +
                   "description" + description +
                   "isPublic" + "1" +
                   "tags" + tags;

      var signature = PhotoMapUtils.getMD5(params);

      this.boundary = this.createBoundary();

      var body = this.getPostValue("api_key",    id)     + 
                 this.getPostValue("async",      "1")    +
                 this.getPostValue("auth_token", token)  +
                 this.getPostValue("description",   description)    +
                 this.getPostValue("isPublic",   "1")    +
                 this.getPostValue("tags",       tags)   +
                 this.getPostValue("api_sig",    signature) +
                 "--" + this.boundary + "\r\n" +
                 "Content-Disposition: form-data; name=\"photo\"; filename=\"" + editInfo.file.leafName + "\"\r\n" +
                 "Content-Type: image/jpeg\r\n" +
                 "\r\n";

      var bodyStream = PhotoMapUtils.createStringInputStream(body);
      var fileStream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance(Components.interfaces.nsIFileInputStream);
      fileStream.init(editInfo.file, 0x01, 0444, null);
      var fileBufStream = Components.classes["@mozilla.org/network/buffered-input-stream;1"].createInstance(Components.interfaces.nsIBufferedInputStream);
      // what is this magic number?
      fileBufStream.init(fileStream, 9000000);
      var endStream  = PhotoMapUtils.createStringInputStream("--" + this.boundary + "--\r\n");

      var uploadStream = Components.classes["@mozilla.org/io/multiplex-input-stream;1"].createInstance(Components.interfaces.nsIMultiplexInputStream);
      uploadStream.appendStream(bodyStream);
      uploadStream.appendStream(fileBufStream);
      uploadStream.appendStream(endStream);

      this.body = uploadStream;

      this.request.onload = function (evt) {
                              fileBufStream.close();
                              bodyStream.close();
                              endStream.close();
                              fileStream.close();
                            };
			                       
   },

   getPostValue : function(key, value) {
      return "--" + this.boundary + "\r\nContent-Disposition: form-data; name=\"" + key + "\"\r\n\r\n" + value + "\r\n";
   },

   createBoundary : function() {
      // make this more random, and make sure it's not in the payload
      return "---------------------------215222648325949";
   },

   getRequestHeaders : function() {
      // do we need our own size?
      return {'Content-Type' : 'multipart/form-data; boundary=' + this.boundary};
   },

   parseResponse : function() {
      // this throws an exception...
      this.checkForErrors();

      // pull the values from xmlresponse
      this.ticketId = this.getStringValue("//ticketid", this.xmlResponse);
   },

   getTicketId : function() {
      return this.ticketId;
   }

}
