//  **** BEGIN LICENSE BLOCK ****
//  Copyright(c) 2005-2006 Adam Judson
//
//
//  PhotoMap
//  - display user contributed photos in a map
//
//  This program is free software; you can redistribute it and/or modify it under
//  the terms of the GNU General Public License as published by the Free
//  Software Foundation; either version 2 of the License, or (at your option)
//  any later version.
//
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
//  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
//  more details.
//
//  You should have received a copy of the GNU General Public License along with
//  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
//  Place, Suite 330, Boston, MA 02111-1307 USA
//
//  **** END LICENSE BLOCK ****


// SearchTags object
function SearchTags(bounds, zoom) {
   this.init(bounds, zoom);
}

SearchTags.key = function(lat, lon) {
   return lat + "x" + lon;
}

SearchTags.prototype = {
   init : function(bounds, zoom) {
      this.bounds = bounds;
      this.zoom = zoom;
      this.deltaX = bounds.maxX - bounds.minX;
      this.deltaY = bounds.maxY - bounds.minY;
   },

   getAll: function(start, end, step) {
      var list = new Array();

      var myStart = PhotoMapUtils.floor(start/step)* step;
      var myEnd = (PhotoMapUtils.floor(end/step) + 1) * step;
      for (myStart; myStart < myEnd; myStart += step) {
         list.push(this.fixFloating(myStart, step));
      }
      return list;
   },

   fixFloating : function(num, step) {
      if (step <= 0.01) {
         return Math.round(num * 100)/100;
      } else if (step <= 0.1) {
         return Math.round(num * 10)/10;
      } else {
         return Math.round(num);
      }
   },

   // make this better
   getSteps : function(step) {
      var list = this.getAll(this.bounds.minX, this.bounds.maxX, step);
      return list.length;
   },

   getAllCombinations : function(lons, lonPrefix, lats, latPrefix) {
      var list = new Array();
      var i, j;
      var latValue;
      var subList;
      for (i = 0; i < lats.length; i++) {
         latValue = latPrefix + "" + lats[i];
         for (j = 0; j < lons.length; j++) {
            subList = [];

            subList.push(latValue);
            if (lats[i] < 0) {
               subList.push("negativeLat");
            }

            subList.push(lonPrefix + "" + lons[j]);
            if (lons[j] < 0) {
               subList.push("negativeLon");
            }

            list.push(subList);
         }
      }
      return list;
   },

   // do we need a bigger step?.
   steps :  {
              "twoDecimal"              :  0.01,
              "oneDecimal"              :  0.1, 
              "short"                   :  1,   
              "approx"                  : 10
            },

   // this is kind of a mess...
   zooms :  { 75    : Photo.WONDER,              
              25    : Photo.MAJOR_LANDMARK,     
              10    : Photo.CAPITOL,            
               5    : Photo.LANDMARK,           
               1    : Photo.BIG_INTERSECTION, 
               0.5  : Photo.INTERSECTION,
               0.1  : Photo.SMALL_INTERSECTION,
               0.01 : Photo.MID_BLOCK          
            },

   getMinType : function() {
      var minType = Math.min(Photo.WONDER, this.zoom);
      return minType;
   },

   getMinTypeForStep : function(step) {
      var minType = Photo.WONDER;
      for (var zoomStep in this.zooms) {
         if ((step)<= (0 + zoomStep)) {
            minType = this.zooms[zoomStep];
         }
      }
      return minType;
   },

   getSearches : function() {
      // return a collection of search tags, covering all necessary searches
      // bounds will determine minimum type
      // that may not be correct
      var minType = this.getMinType();
   
      var searches = new Array();
      var searchTags;

      var step, prefix, type, minStepType;
      for (prefix in this.steps) {
         step = this.steps[prefix];

         // if we generate less than 4 x 4 searches, then we use it
         if (this.getSteps(step) < 5) {
            minStepType = this.getMinTypeForStep(step);
            var lonList = this.getAll(this.bounds.minX, this.bounds.maxX, step);
            var latList = this.getAll(this.bounds.minY, this.bounds.maxY, step);
            var combinations = this.getAllCombinations(lonList, prefix + "Lon:", latList, prefix + "Lat:");
            for (var i = 0; i < combinations.length; i++) {
               searchTags = combinations[i];
               searchTags.type = minType;
               searches.push(searchTags);
            }
            break;
         }
      }
      return searches;
   }
}

